/** @file system.c 
*   @brief System Driver Source File
*   @date 15.January.2012
*   @version 3.00.00
*
*   This file contains:
*   - API Functions
*   .
*   which are relevant for the System driver.
*/

/* (c) Texas Instruments 2012, All rights reserved. */

#include "sys_vim.h"
#include "sys_common.h"


#pragma DATA_SECTION(VIM_TABLE, ".vim_table");

const uint32_t VIM_TABLE[48] = 
{
 
 (uint32_t) &_NMI,	/* Channel 0 */

#if VIM_CONFIG
 
 (uint32_t) &Dummy_ISR,   			/* Channel 1 */


 (uint32_t) &esmLowInterrupt,	/* Channel 2 */


 (uint32_t) &Dummy_ISR,			/* Channel 3 */


 (uint32_t) &Dummy_ISR,			/* Channel 4 */


 (uint32_t) &Dummy_ISR,			/* Channel 5 */


 (uint32_t) &Dummy_ISR,			/* Channel 6 */


 (uint32_t) &Dummy_ISR,                     /* Channel 7 */


 (uint32_t) &Dummy_ISR,                    /* Channel 8 */


 (uint32_t) &Dummy_ISR,                    /* Channel 9 */


 (uint32_t) &Dummy_ISR,                     /* Channel 10 */

 (uint32_t) &Dummy_ISR,                     /* Channel 11 */


 (uint32_t) &Dummy_ISR,                     /* Channel 12 */


 (uint32_t) &hetHighLevelInterrupt,       /* Channel 13 */


 (uint32_t) &hetLowLevelInterrupt,       /* Channel 14 */


 (uint32_t) &Dummy_ISR,                     /* Channel 15 */


 (uint32_t) &Dummy_ISR,                     /* Channel 16 */


 (uint32_t) &Dummy_ISR,                     /* Channel 17 */


 (uint32_t) &Dummy_ISR,                     /* Channel 18 */


 (uint32_t) &Dummy_ISR,                     /* Channel 19 */


 (uint32_t) &Dummy_ISR,                     /* Channel 20 */


 (uint32_t) &Dummy_ISR,                     /* Channel 21 */


 (uint32_t) &Dummy_ISR,                     /* Channel 22 */


 (uint32_t) &Dummy_ISR,                     /* Channel 23 */


 (uint32_t) &Dummy_ISR,                     /* Channel 24 */


 (uint32_t) &Dummy_ISR,                     /* Channel 25 */


 (uint32_t) &Dummy_ISR,                     /* Channel 26 */


 (uint32_t) &Dummy_ISR,                     /* Channel 27 */


 (uint32_t) &Dummy_ISR,                     /* Channel 28 */


 (uint32_t) &Dummy_ISR,                     /* Channel 29 */


 (uint32_t) &Dummy_ISR,                     /* Channel 30 */


 (uint32_t) &Dummy_ISR,                     /* Channel 31 */


 (uint32_t) &Dummy_ISR,


 (uint32_t) &Dummy_ISR,


 (uint32_t) &Dummy_ISR,


 (uint32_t) &Dummy_ISR,


 (uint32_t) &Dummy_ISR,


 (uint32_t) &Dummy_ISR,


 (uint32_t) &Dummy_ISR,


 (uint32_t) &Dummy_ISR,                          /* Channel 39 */


 (uint32_t) &Dummy_ISR,


 (uint32_t) &Dummy_ISR,


 (uint32_t) &Dummy_ISR,


 (uint32_t) &Dummy_ISR,


 (uint32_t) &Dummy_ISR,


 (uint32_t) &Dummy_ISR,


 (uint32_t) &Dummy_ISR,


 (uint32_t) &Dummy_ISR,                         /* Channel 47 */


#endif

};



/** @fn void Dummy_ISR(void)
*   @brief ISR for undefined interrupts
*
*   The undefined interrupts will have this function as an ISR.
*
*/


void Dummy_ISR(void)
{
/* USER CODE BEGIN (0) */
/* USER CODE END */
  for(;;);                                     /* .... wait here */
}


/** @fn void captureEventSource0(void)
*   @brief Capture Event Source 0
*
*   Function to chose a interrupt channel to Capture RTI counter capture 0.
*
*/

void captureEventSource0()  /* Capture Event Source 0 -> RTI CAPCNTR0 */
{
    vimREG->CAPEVTSRC0 = 0;
/* USER CODE BEGIN (1) */
/* USER CODE END */
}




/** @fn void captureEventSource0(void)
*   @brief Capture Event Source 0
*
*   Function to chose a interrupt channel to Capture RTI counter capture 1 .
*
*/

void captureEventSource1()  /* Capture Event Source 1 -> RTI CAPCNTR1 */
{
    vimREG->CAPEVTSRC1 = 0;
/* USER CODE BEGIN (2) */
/* USER CODE END */
}





/** @fn void nestEnable(void)
*   @brief Enable nested interrupts
*
*   Function to enable interrupt nesting upto 8 levels.
*
*/

void nestEnable(void)
{
    vimREG->NESTCTL = 0x0A;
}



/** @fn void vimCfg(void)
*   @brief VIM Configuration
*
*   Function to configure VIM.
*
*/

void vimCfg(void)
{	
       
	/* set NMI/ISR priorities */
    vimREG->NMIPR0 =  (1U <<  2U)
                    | (0U <<  3U)
                    | (0U <<  4U)
                    | (0U <<  5U)
                    | (0U <<  6U)
                    | (0U <<  7U)
                    | (0U <<  8U)
                    | (0U <<  9U)
                    | (0U << 10U)
                    | (0U << 11U)
                    | (0U << 12U)
                    | (0U << 13U)
                    | (0U << 14U)
                    | (0U << 15U)
                    | (0U << 16U)
                    | (0U << 17U)
                    | (0U << 18U)
                    | (0U << 19U)
                    | (0U << 20U)
                    | (0U << 21U)
                    | (0U << 22U)
                    | (0U << 23U)
                    | (0U << 24U)
                    | (0U << 25U)
                    | (0U << 26U)
                    | (0U << 27U)
                    | (0U << 28U)
                    | (0U << 29U)
                    | (0U << 30U)
                    | (0U << 31U);

    vimREG->NMIPR1 =  0U
                    | (0U <<  1U)
                    | (0U <<  2U)
                    | (0U <<  3U)
                    | (0U <<  4U)
                    | (0U <<  5U)
                    | (0U <<  6U)
                    | (0U <<  7U)
                    | (0U <<  8U)
                    | (0U <<  9U)
                    | (0U << 10U)
                    | (0U << 11U)
                    | (0U << 12U)
                    | (0U << 13U)
                    | (0U << 14U)
                    | (0U << 15U);

    /* enable interrupts */
    vimREG->REQMASKSET0 = 
                          (1U <<  2U)
                        | (0U <<  3U)
                        | (0U <<  4U)
                        | (0U <<  5U)
                        | (0U <<  6U)
                        | (0U <<  7U)
                        | (0U <<  8U)
                        | (0U <<  9U)
                        | (0U << 10U)
                        | (0U << 11U)
                        | (0U << 12U)
                        | (1U << 13U)
                        | (1U << 14U)
                        | (0U << 15U)
                        | (0U << 16U)
                        | (0U << 17U)
                        | (0U << 18U)
                        | (0U << 19U)
                        | (0U << 20U)
                        | (0U << 21U)
                        | (0U << 22U)
                        | (0U << 23U)
                        | (0U << 24U)
                        | (0U << 25U)
                        | (0U << 26U)
                        | (0U << 27U)
                        | (0U << 28U)
                        | (0U << 29U)
                        | (0U << 30U)
                        | (0U << 31U);

    vimREG->REQMASKSET1 = 0U
                        | (0U << 1U)
                        | (0U << 2U)
                        | (0U << 3U)
                        | (0U << 4U)
                        | (0U << 5U)
                        | (0U << 6U)
                        | (0U << 7U)
                        | (0U << 8U)
                        | (0U << 9U)
                        | (0U << 10U)
                        | (0U << 11U)
                        | (0U << 12U)
                        | (0U << 13U)
                        | (0U << 14U)
                        | (0U << 15U);

    
    
        /* To RTI CAPCNTR0*/
        captureEventSource0(); 

        /* To RTI CAPCNTR1*/
        captureEventSource1(); 
    
/* USER CODE BEGIN (3) */
/* USER CODE END */
}





/*- End of sys_startup.c ----------------------------------------------*/




